#!/usr/bin/perl

#┌─────────────────────────────────
#│ Web Scheduler : schedule.cgi - 2022/02/06
#│ copyright (c) KentWeb, 1997-2022
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  WebScheduler v4.0 - 2022/02/11
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────
# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use Encode;
use lib "./lib";
use CGI::Minimal;
use Calendar::Japanese::Holiday;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# 主要処理
if ($in{form}) { form_date(); }
if ($in{mode} eq 'put_log') { put_log(); }
if ($in{mode} eq 'icon') { icon_page(); }

list_calen();

#-----------------------------------------------------------
#  リスト表示
#-----------------------------------------------------------
sub list_calen {
	# 年月引数解釈
	if ($in{date} =~ m|^(\d{4})/(\d{2})|) {
		$in{y} = $1;
		$in{m} = $2;
	} elsif ($in{y} && $in{m}) {
		$in{y} =~ s/\D//g;
		$in{m} =~ s/\D//g;
	}
	
	# 本日
	my ($mday,$mon,$year) = (localtime())[3..5];
	$year += 1900;
	$mon  = sprintf("%02d",$mon + 1);
	$mday = sprintf("%02d",$mday);
	my $today;
	if ($in{y} !~ /^\d{4}$/ || $in{m} > 12 || $in{m} < 1) {
		$in{y} = $year;
		$in{m} = $mon;
		$today = $mday;
	} elsif ($in{y} == $year && $in{m} == $mon) {
		$today = $mday;
	}
	
	# 前月/翌月ボタン
	my ($btn_back,$btn_next);
	if ($in{m} == 1) {
		my $y = sprintf("%04d",$in{y}-1);
		$btn_back = "y=$y&amp;m=12";
		$btn_next = "y=$in{y}&amp;m=02";
	} elsif ($in{m} == 12) {
		my $y = sprintf("%04d",$in{y}+1);
		$btn_back = "y=$in{y}&amp;m=11";
		$btn_next = "y=$y&amp;m=01";
	} else {
		my $mb = sprintf("%02d",$in{m}-1);
		my $mn = sprintf("%02d",$in{m}+1);
		$btn_back = "y=$in{y}&amp;m=$mb";
		$btn_next = "y=$in{y}&amp;m=$mn";
	}
	
	# 末日
	my $last = last_day($in{y},$in{m});
	
	# 初日の曜日
	my $week = get_week($in{y},$in{m},1);
	
	# 年月データ認識
	my $log = sprintf("%04d%02d",$in{y},$in{m});
	my (%tim,%act,%col,%ico);
	open(IN,"$cf{logdir}/$log.dat");
	while(<IN>) {
		chomp;
		my ($no,$day,$fr1,$fr2,$to1,$to2,$act,$ex,$col,$ico) = split(/<>/);
		
		# 時間
		my $time = $to1 eq '' ? "$fr1:$fr2～" : "$fr1:$fr2～$to1:$to2";
		$tim{$day} .= "<br>\n" if (defined($tim{$day}));
		#$tim{$day} .= $time;
		if ($fr1 eq '') { 
		$tim{$day} .= "開始時間未定";
		} else {
		$tim{$day} .= $time;
		}
		
		# 内容
		my $css_col = "color:$col";
		$act{$day} .= "<br />\n" if (defined($act{$day}));
		$act = qq|<img src="$cf{iconurl}/$ico" style="vertical-align: middle"> $act| if ($ico);
		$act = qq|<span style="$css_col"> $act</span>| if ($col);
		$act .= qq|<a href="$cf{uplurl}/$no.$ex" target="_blank"><img src="$cf{cmnurl}/$cf{icon}{$ex}" alt="$ex" class="ico" /></a>| if ($ex);

		$act{$day} .= $act;
		$col{$day} .= $col;
		$ico{$day} .= $ico;
	}
	close(IN);
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/list.html") or error('open err: list.html');
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置換え
	$tmpl =~ s/!homepage!/$cf{homepage}/g;
	$tmpl =~ s/!title!/$cf{title}/g;
	$tmpl =~ s/!year!/$in{y}/g;
	$tmpl =~ s/!mon!/$in{m}/g;
	$tmpl =~ s/!(\w+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s|!btn-back!|<a href="$cf{index_cgi}?$btn_back"><img src="$cf{cmnurl}/back.png" alt="前月"></a>|g;
	$tmpl =~ s|!btn-next!|<a href="$cf{index_cgi}?$btn_next"><img src="$cf{cmnurl}/next.png" alt="翌月"></a>|g;
	$tmpl =~ s|!btn-this!|<a href="$cf{index_cgi}"><img src="$cf{cmnurl}/now.png" alt="当月"></a>|g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon">|g;
	$tmpl =~ s|!icon/(\w+\.png)!|$cf{iconurl}/$1|g;
	$tmpl =~ s|!icon/(\w+\.gif)!|$cf{iconurl}/$1|g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# 祝日定義
	my $hdays = eval "getHolidays($in{y},$in{m},1);";
	my %hol;
	while ( my ($key,$val) = each %{$hdays} ) {
		$hol{$key} = encode('utf8',$val);
	}
	
	# ヘッダー表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 内容表示
	my $w = $week;
	foreach my $i (1 .. $last) {
		
		# 日を２桁化
		my $d = sprintf("%02d",$i);
		
		# 祝日チェック
		my $css_week;
		if (defined($hol{$d})) {
			$css_week = "color:$cf{wkcol}[7];font-weight: bolder;";
			#$act{$i} = qq|<span style="$css_week;font-size:smaller">$hol{$d}</span><br />$act{$i}|;
			#$tim{$i} = "&nbsp;<br />$tim{$i}";
			$hol{$i} = qq|<span style="$css_week;font-size:9.5pt">$hol{$d}</span>|;
		} else {
			$css_week = "color:$cf{wkcol}[$w];";
		}
		
		# 本日
		my $css_back = "text-align:left;";
		if ($today == $i) {
			$css_back .= "background:$cf{wkcol}[8];";
			$css_week .= "background:$cf{wkcol}[8];";
		}
		
		# 文字置換え
		my $tmp = $loop;
		$tmp =~ s/!css-week!/$css_week/g;
		$tmp =~ s/!css-back!/$css_back/g;
		$tmp =~ s|!day!|<a href="$cf{index_cgi}?form=$in{y}$in{m}$d" style="$css_week">$i</a>|g;
		$tmp =~ s/!week!/$cf{week}->[$w]/g;
		$tmp =~ s/!time!/$tim{$i} ne '' ? $tim{$i} : '<br>'/eg;
		$tmp =~ s/!action!/$act{$i} ne '' ? $act{$i} : '<br>'/eg;
		$tmp =~ s/!holiday!/$hol{$i} ne '' ? $hol{$i} : '<br>'/eg;
		print $tmp;
		
		# 曜日フラグ
		$w = $w == 6 ? 0 : ++$w;
	}
	
	# フッター表示
	footer($foot);
}

#-----------------------------------------------------------
#  書き込みフォーム
#-----------------------------------------------------------
sub form_date {
	# 引数チェック
	my ($y,$m,$d) = $in{form} =~ /^(\d{4})(\d{2})(\d{2})$/
			? ($1,$2,$3)
			: error('不正な要求です');
	
	# 該当データ抽出
	my @log;
	open(IN,"$cf{logdir}/$y$m.dat");
	while(<IN>) {
		chomp;
		my ($no,$day,$fr1,$fr2,$to1,$to2,$act,$ex,$col,$ico) = split(/<>/);
		
		if ($day == $d) { push(@log,$_); }
	}
	close(IN);
	
	# 曜日
	my $week = get_week($y,$m,$d);
	my $date = "$y年$m月$d日（$cf{week}->[$week]）";
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/form.html") or error('open err: form.html');
	my $tmpl = join('', <IN>);
	close(IN);

	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my ($color,$col);
	foreach (0 .. $#col) {
		if ($col == $_) {
			$color .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$color .= qq|<input type="radio" name="color" value="$_">|;
		}
		$color .= qq|<span style="color:$col[$_]">●</span>\n|;
	}

	# アイコンプルダウン
	my $op_icon;
	foreach (0 .. $#{$cf{general_ico}}) {
		my ($fnam,$nam) = split(/,/, $cf{general_ico}->[$_]);
		if ($_ == $fnam) {
			$op_icon .= qq|<option value="$_" selected>$nam\n|;
		} else {
			$op_icon .= qq|<option value="$_">$nam\n|;
		}
	}
	
	# 文字置換え
	$tmpl =~ s/!title!/$cf{title}/g;
	$tmpl =~ s/!(\w+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!date!/$date/g;
	$tmpl =~ s/!color!/$color/g;
	$tmpl =~ s/<!-- op_icon -->/$op_icon/g;
	$tmpl =~ s/!y!/$y/g;
	$tmpl =~ s/!m!/$m/g;
	$tmpl =~ s/!d!/$d/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon">|g;
	
	# テンプレート分割
		my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# 新規フォーム
	$head =~ s/<!-- op_from_h -->/op_h()/eg;
	$head =~ s/<!-- op_from_m -->/op_m()/eg;
	$head =~ s/<!-- op_to_h -->/op_h()/eg;
	$head =~ s/<!-- op_to_m -->/op_m()/eg;
	
	# ヘッダー表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# メンテ用フォーム
	my $i = 0;
	foreach (@log) {
		$i++;
		my ($no,$day,$fr1,$fr2,$to1,$to2,$act,$ex,$color,$icon) = split(/<>/);
		
	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my $editcolor;
	foreach (0 .. $#col) {
		if ($col[$_] eq $color) {
			$editcolor .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$editcolor .= qq|<input type="radio" name="color" value="$_">|;
		}
		$editcolor .= qq|<span style="color:$col[$_]">●</span>\n|;
	}

	# アイコンプルダウン
	my $editop_icon;
	foreach (0 .. $#{$cf{general_ico}}) {
		my ($fnam,$nam) = split(/,/, $cf{general_ico}->[$_]);
		if ($icon eq $fnam) {
			$editop_icon .= qq|<option value="$_" selected>$nam\n|;
		} else {
			$editop_icon .= qq|<option value="$_">$nam\n|;
		}
	}
		# 添付有り
		my $upf;
		if ($ex) {
			$upf .= qq|&nbsp; [<a href="$cf{uplurl}/$no.$ex" target="_blank">添付</a>]\n|;
			$upf .= qq|<input type="checkbox" name="del" value="1">削除\n|;
		}
		
		my $tmp = $loop;
		$tmp =~ s/form-ttl-new/form-ttl-edit/g;
		$tmp =~ s/!form-num!/$i/g;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!action!/$act/g;
		$tmp =~ s/!editcolor!/$editcolor/g;
		$tmp =~ s/<!-- editop_icon -->/$editop_icon/g;
		$tmp =~ s/<!-- op_from_h -->/op_h($fr1)/eg;
		$tmp =~ s/<!-- op_from_m -->/op_m($fr2)/eg;
		$tmp =~ s/<!-- op_to_h -->/op_h($to1)/eg;
		$tmp =~ s/<!-- op_to_m -->/op_m($to2)/eg;
		$tmp =~ s/<!-- upfile -->/$upf/;
		print $tmp;
	}
	
	# フッター表示
	footer($foot);
}

#-----------------------------------------------------------
#  プルダウン：時間
#-----------------------------------------------------------
sub op_h {
	my $h = shift;
	
	my $ret;
	for my $i ( 0 .. 23 ) {
		$i = sprintf("%02d",$i);
		
		if ($h eq $i) {
			$ret .= qq|<option value="$i" selected="selected">$i</option>\n|;
		} else {
			$ret .= qq|<option value="$i">$i</option>\n|;
		}
	}
	$ret;
}

#-----------------------------------------------------------
#  プルダウン：分
#-----------------------------------------------------------
sub op_m {
	my $m = shift;
	
	my $ret;
	for ( my $i = 0; $i <= 55; $i += 5 ) {
		$i = sprintf("%02d",$i);
		
		if ($m eq $i) {
			$ret .= qq|<option value="$i" selected="selected">$i</option>\n|;
		} else {
			$ret .= qq|<option value="$i">$i</option>\n|;
		}
	}
	$ret;
}

#-----------------------------------------------------------
#  ログ更新
#-----------------------------------------------------------
sub put_log {
	# パスワード認証
	error('パスワードが認証できません') if ($in{pwd} ne $cf{password});
	
	# 入力チェック
	if ($in{y} !~ /^\d{4}$/ || $in{m} > 12 || $in{m} < 1) {
		error('不正な要求です');
	}
	my $err;
	#if ($in{from_h} eq '' || $in{from_m} eq '') { $err .= "開始時間が未選択です<br>"; }
	if ($in{action} eq '') { $err .= "内容が未入力です<br>"; }
	error($err) if ($err);
	
	# ログ用の「日」
	my $d = $in{d};
	$d =~ s/^0//;

	# アイコン
	my $icon = (split(/,/, $cf{general_ico}->[$in{icon}]))[0];

	# 文字色
	my $color = (split(/\s+/, $cf{colors}))[$in{color}];
	
	# 新規の場合：採番
	my $new;
	if ($in{num} eq 'new') {
		open(DAT,"+< $cf{numfile}") or error("open err: $cf{numfile}");
		$new = <DAT> + 1;
		seek(DAT, 0, 0);
		print DAT $new;
		truncate(DAT, tell(DAT));
		close(DAT);
	}
	
	# アップロード
	my $upno = $new ? $new : $in{num};
	my $ext = upload_file($upno) if ($in{upfile});
	
	# 当月データがない場合
	if (! -f "$cf{logdir}/$in{y}$in{m}.dat") {
		open(DAT,"+> $cf{logdir}/$in{y}$in{m}.dat") or error("write err: $in{y}$in{m}.dat");
		close(DAT);
		chmod(0666,"$cf{logdir}/$in{y}$in{m}.dat");
	}
	# 書き込みオープン
	my (@tmp1,@tmp2,@log);
	open(DAT,"+< $cf{logdir}/$in{y}$in{m}.dat");
	while(<DAT>) {
		chomp;
		my ($no,$day,$fr1,$fr2,$to1,$to2,$act,$ex,$col,$ico) = split(/<>/);
		
		# 編集のとき
		if ($in{job_edit} && $in{num} == $no) {
			
			# 添付削除
			if ($ex && $in{del}) {
				unlink("$cf{upldir}/$no.$ex");
				$ex = '';
			}
			
			# 添付有り
			if ($ext) {
				unlink("$cf{upldir}/$no.$ex") if ($ex && $ex ne $ext);
				$ex = $ext;
			}
			$_ = "$no<>$d<>$in{from_h}<>$in{from_m}<>$in{to_h}<>$in{to_m}<>$in{action}<>$ex<>$color<>$icon<>";
			$day = $d;
			$fr1 = $in{from_h};
			$fr2 = $in{from_m};
			
		# 削除のとき
		} elsif ($in{job_dele} && $in{num} == $no) {
			next;
		}
		
		push(@tmp1,$day);
		push(@tmp2,"$fr1$fr2");
		push(@log,"$_\n");
	}
	
	# 新規の場合
	if ($in{num} eq 'new') {
		push(@tmp1,$in{d});
		push(@tmp2,"$in{from_h}$in{from_m}");
		push(@log,"$new<>$d<>$in{from_h}<>$in{from_m}<>$in{to_h}<>$in{to_m}<>$in{action}<>$ext<>$color<>$icon<>\n");
	}
	
	# 日時順にソートする
	@log = @log[sort{$tmp1[$a] <=> $tmp1[$b] || $tmp2[$a] <=> $tmp2[$b]} 0 .. $#tmp1];
	
	# データ更新
	seek(DAT, 0, 0);
	print DAT @log;
	truncate(DAT, tell(DAT));
	close(DAT);
}

#-----------------------------------------------------------
#  フォームデコード
#-----------------------------------------------------------
sub parse_form {
	my $cgi = shift;
	
	my %in;
	foreach ( $cgi->param() ) {
		my $val = $cgi->param($_);
		
		if ($_ ne 'upfile') {
			# 無効化
			$val =~ s/&/&amp;/g;
			$val =~ s/</&lt;/g;
			$val =~ s/>/&gt;/g;
			$val =~ s/"/&quot;/g;
			$val =~ s/'/&#39;/g;
			$val =~ s/[\r\n]//g;
		}
		$in{$_} = $val;
	}
	return %in;
}

#-----------------------------------------------------------
#  アイコン一覧
#-----------------------------------------------------------
sub icon_page {
	# 画像サイズ再定義
	$cf{max_img_w} = 40;
	$cf{max_img_h} = 40;

	# テンプレート認識
	open(IN,"$cf{tmpldir}/icon.html") or &error("open err: icon.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot);
	if ($tmpl =~ /(.+)<!-- photo_begin -->(.+)<!-- photo_end -->(.+)/s) {
		($head,$loop,$foot) = ($1,$2,$3);
	} else {
		&error("テンプレートが不正です");
	}

	# 文字置換
	foreach ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!cmnurl!/$cf{cmnurl}/g;
	}

	# 画面展開
	print "Content-type: text/html\n\n";
	print $head;

	foreach (0 .. $#{$cf{general_ico}}) {
		my ($ico,$cap) = split(/,/, $cf{general_ico}->[$_]);

		my $tmp = $loop;
		$tmp =~ s|!image!|<img src="$cf{iconurl}/$ico">|g;
		$tmp =~ s/!caption!/$cap/g;
		print $tmp;
	}

	# フッタ
	print $foot;
	exit;
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;
	
	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<p style="margin-top:2em;text-align:center;font-family:Verdana,Helvetica,Arial;font-size:10px;">
	- <a href="https://www.kent-web.com/">$cf{version}</a> -<br />
	++ <a href="https://www.transformsite.com/" target="_top">Edited by TRANSFORM</a> ++
</p>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  エラー画面
#-----------------------------------------------------------
sub error {
	my $err = shift;
	
	open(IN,"$cf{tmpldir}/error.html") or die;
	my $tmpl = join('', <IN>);
	close(IN);
	
	$tmpl =~ s/!error!/$err/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  添付アップロード
#-----------------------------------------------------------
sub upload_file {
	my $no = shift;
	
	# 拡張子取得
	my $ext = $cgi->param_filename('upfile') =~ /\.(docx?|xlsx?|pptx?|pdf|zip)$/i
			? $1
			: error("拡張子不明");
	$ext =~ tr/A-Z/a-z/;
	
	# 添付ファイル定義
	my $upfile = "$cf{upldir}/$no.$ext";
	
	# アップロード
	open(UP,"+> $upfile") or error("up err: $upfile");
	binmode(UP);
	print UP $in{upfile};
	close(UP);
	
	# パーミッションを666へ
	chmod(0666,$upfile);
	
	# 結果を返す
	return $ext;
}

#-----------------------------------------------------------
#  月の末日
#-----------------------------------------------------------
sub last_day {
	my ($yy,$mm) = @_;
	
	return (31,28,31,30,31,30,31,31,30,31,30,31) [$mm - 1]
		+ ($mm == 2 && (($yy % 4 == 0 && $yy % 100 != 0) || $yy % 400 == 0));
}

#-----------------------------------------------------------
#  週算出
#-----------------------------------------------------------
sub get_week {
	my ($yy,$mm,$dd) = @_;
	
	if ($mm == 1 || $mm == 2) {
		$yy--;
		$mm += 12;
	}
	int( $yy + int($yy / 4) - int($yy / 100) + int($yy / 400) + int((13 * $mm + 8) / 5) + $dd ) % 7;
}

